package com.kakao.auth.authorization.accesstoken;


import com.kakao.auth.StringSet;
import com.kakao.network.response.ResponseBody;
import com.kakao.test.common.KakaoTestCase;

import org.json.JSONException;
import org.json.JSONObject;
import org.junit.Before;
import org.junit.Test;
import static org.junit.Assert.*;

/**
 * This class mocks KakaoSDK, Utility, SharedPreferenceCache, ResponseBody for solely unit-testing AccessTokenImpl class.
 * @author kevin.kang
 * Created by kevin.kang on 16. 8. 11..
 */
public class AccessTokenImplTest extends KakaoTestCase {

    private AccessTokenImpl tokenInfo;
    @Before
    public void setup() {
        super.setup();
    }

    @Test
    public void testCreateEmptyToken() {
        tokenInfo = (AccessTokenImpl) TestAccessTokenFactory.createEmptyAccessToken();
        assertNotNull(tokenInfo);
        assertNull(tokenInfo.getAccessToken());
        assertNull(tokenInfo.getRefreshToken());
        assertFalse(tokenInfo.hasValidAccessToken());
        assertFalse(tokenInfo.hasRefreshToken());
    }

    @Test
    public void testCreateFromCache() {
        tokenInfo = new AccessTokenImpl(new TestAccessToken());
        assertTrue(tokenInfo.hasValidAccessToken());
        assertTrue(tokenInfo.hasRefreshToken());
        assertEquals("access_token", tokenInfo.getAccessToken());
        assertEquals("refresh_token", tokenInfo.getRefreshToken());
    }

    @Test
    public void createFromResponse() {
        try {
            AccessToken tokenInfo = new AccessTokenImpl(createTokenResponseBody(true));

            assertEquals("access_token", tokenInfo.getAccessToken());
            assertEquals("refresh_token", tokenInfo.getRefreshToken());
            assertTrue(tokenInfo.hasValidAccessToken());
            assertTrue(tokenInfo.hasRefreshToken());
            assertNotNull(tokenInfo.accessTokenExpiresAt());
            assertNotNull(tokenInfo.refreshTokenExpiresAt());

            ResponseBody body = createTokenResponseBody(false);
            tokenInfo = new AccessTokenImpl(body);
            assertEquals("access_token", tokenInfo.getAccessToken());
            assertNull(tokenInfo.getRefreshToken());
            assertTrue(tokenInfo.hasValidAccessToken());
            assertFalse(tokenInfo.hasRefreshToken());
            assertNotNull(tokenInfo.refreshTokenExpiresAt());


        } catch (ResponseBody.ResponseBodyException e) {
            e.printStackTrace();
        }
    }

    @Test
    public void createFromString() {
        String string = createTokenResponseBody(true).toString();
        AccessToken tokenInfo = new AccessTokenImpl(string);
        assertEquals("access_token", tokenInfo.getAccessToken());
        assertEquals("refresh_token", tokenInfo.getRefreshToken());
        assertTrue(tokenInfo.hasValidAccessToken());
        assertTrue(tokenInfo.hasRefreshToken());


        ResponseBody body = createTokenResponseBody(false);
        tokenInfo = new AccessTokenImpl(body);
        assertEquals("access_token", tokenInfo.getAccessToken());
        assertNull(tokenInfo.getRefreshToken());
        assertTrue(tokenInfo.hasValidAccessToken());
        assertFalse(tokenInfo.hasRefreshToken());
    }

    @Test
    public void createFromResponseWithException() {
    }

    ResponseBody createTokenResponseBody(boolean hasRefreshToken) {
        try {
            JSONObject jsonObject = new JSONObject();
            jsonObject.put(StringSet.access_token, "access_token");
            jsonObject.put(StringSet.expires_in, 43199);
            if (hasRefreshToken) {
                jsonObject.put(StringSet.refresh_token, "refresh_token");
            }
            return new ResponseBody(jsonObject);
        } catch (JSONException|ResponseBody.ResponseBodyException e) {
            return null;
        }
    }
}
