package com.kakao.kakaolink.v2;

import android.app.Activity;

import com.kakao.kakaolink.BuildConfig;
import com.kakao.kakaolink.R;
import com.kakao.kakaolink.v2.mocks.TestKakaoLinkIMageService;
import com.kakao.kakaolink.v2.mocks.TestKakaoLinkCore;
import com.kakao.kakaolink.v2.network.KakaoLinkCore;
import com.kakao.kakaolink.v2.network.KakaoLinkImageService;
import com.kakao.network.ErrorResult;
import com.kakao.network.NetworkService;
import com.kakao.network.callback.ResponseCallback;
import com.kakao.test.common.KakaoTestCase;

import org.junit.After;
import org.junit.Assert;
import org.junit.Before;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.mockito.Mockito;
import org.robolectric.Robolectric;
import org.robolectric.RobolectricTestRunner;
import org.robolectric.annotation.Config;
import org.robolectric.shadows.ShadowLog;
//import org.robolectric.util.concurrent.RoboExecutorService;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * @author kevin.kang. Created on 2016. 11. 28..
 */

@RunWith(RobolectricTestRunner.class)
@Config(constants = BuildConfig.class, sdk = 25)
public class KakaoLinkServiceTest extends KakaoTestCase {
    private Activity activity;
    private final List<String> transcript = new ArrayList<>();
    private KakaoLinkService service;
    private KakaoLinkCore linkService;
    private KakaoLinkImageService imageService;
    private NetworkService networkService;
    private ResponseCallback<KakaoLinkResponse> callback;

    @Before
    public void setup() {
        super.setup();
        activity = Mockito.mock(Activity.class);
        linkService = Mockito.spy(new TestKakaoLinkCore());
        imageService = Mockito.spy(new TestKakaoLinkIMageService());
        networkService = Mockito.spy(new TestNetworkService());
        this.service = Mockito.spy(new KakaoLinkService(linkService, imageService, networkService));
        callback = new ResponseCallback<KakaoLinkResponse>() {
            @Override
            public void onFailure(ErrorResult errorResult) {
                ShadowLog.e("error", errorResult.toString());
                transcript.add("failure");
            }

            @Override
            public void onSuccess(KakaoLinkResponse result) {
                transcript.add("success");
            }
        };
        Robolectric.getBackgroundThreadScheduler().pause();
    }

    @After
    public void after() {
        transcript.clear();
    }

    @Test
    public void testSend() {
        Mockito.doReturn(true).when(linkService).isAvailable(activity);
        Map<String, String> templateArgs = new HashMap<String, String>();
        templateArgs.put("${iphoneAppParam}", "key1=value1");
        service.sendCustom(activity, "91", templateArgs, callback);
        Assert.assertTrue(transcript.contains("success"));
    }

    @Test
    public void testSendWithLowerTalkVersion() {
        Mockito.doReturn(false).when(linkService).isAvailable(activity);
        Mockito.doReturn("com_kakao_alert_install_kakaotalk").when(activity).getString(R.string.com_kakao_alert_install_kakaotalk);

        Map<String, String> templateArgs = new HashMap<>();
        templateArgs.put("${iphoneAppParam}", "key1=value1");

        service.sendCustom(activity, "91", templateArgs, callback);
        Assert.assertTrue(transcript.contains("failure"));
    }
}
