package com.kakao.kakaolink.v2.network;

import android.content.Context;
import android.content.Intent;
import android.net.Uri;

import com.kakao.kakaolink.R;
import com.kakao.kakaolink.internal.KakaoTalkLinkProtocol;
import com.kakao.kakaolink.v2.KakaoLinkTestHelper;
import com.kakao.kakaolink.v2.mocks.TestKakaoUtilService;
import com.kakao.test.common.KakaoTestCase;
import com.kakao.util.IConfiguration;
import com.kakao.util.KakaoUtilService;
import com.kakao.util.exception.KakaoException;
import com.kakao.util.helper.CommonProtocol;

import org.json.JSONException;
import org.json.JSONObject;
import org.junit.Assert;
import org.junit.Test;
import org.mockito.Mockito;
import org.robolectric.RuntimeEnvironment;

/**
 * @author kevin.kang. Created on 2018. 1. 2..
 */

public class DefaultKakaoLinkCoreTest extends KakaoTestCase {
    private DefaultKakaoLinkCore core;
    private IConfiguration configuration;
    private Context context;

    @Override
    public void setup() {
        super.setup();
        KakaoUtilService utilService = new TestKakaoUtilService();
        core = new DefaultKakaoLinkCore(utilService);
        context = Mockito.spy(RuntimeEnvironment.application.getApplicationContext());
        configuration = utilService.getAppConfiguration(context);
        Mockito.doReturn("mocked message").when(context).getString(R.string.com_kakao_alert_uri_too_long);
    }

    @Test
    public void kakaoLinkIntent() {
        Intent intent = core.kakaoLinkIntent(context, "app_key", KakaoLinkTestHelper.createKakaoLinkResponse("1234"));
        Uri uri = intent.getData();
        Assert.assertNotNull(uri);
        Assert.assertEquals(KakaoTalkLinkProtocol.LINK_SCHEME, uri.getScheme());
        Assert.assertEquals(KakaoTalkLinkProtocol.LINK_AUTHORITY, uri.getHost());
        Assert.assertEquals(KakaoTalkLinkProtocol.LINK_VERSION_40, uri.getQueryParameter(KakaoTalkLinkProtocol.LINKVER));
        Assert.assertEquals("app_key", uri.getQueryParameter(KakaoTalkLinkProtocol.APP_KEY));
        Assert.assertEquals(KakaoLinkTestHelper.createMockRequestConfiguration().getAppVer(), uri.getQueryParameter(KakaoTalkLinkProtocol.APP_VER));
        Assert.assertEquals("1234", uri.getQueryParameter(KakaoTalkLinkProtocol.TEMPLATE_ID));
        Assert.assertNotNull(uri.getQueryParameter(KakaoTalkLinkProtocol.TEMPLATE_ARGS));
        Assert.assertNotNull(uri.getQueryParameter(KakaoTalkLinkProtocol.TEMPLATE_JSON));
    }

    @Test
    public void kakaoLinkIntentWithLongUri() {
        try {
            core.kakaoLinkIntent(context, "app_key", KakaoLinkTestHelper.createLongKakaoLinkResponse("1234"));
            Assert.fail("Long uri should throw KakaoException with URI_LENGTH_EXCEEDED error type.");
        } catch (KakaoException e) {
            Assert.assertEquals(KakaoException.ErrorType.URI_LENGTH_EXCEEDED, e.getErrorType());
        }
    }

    @Test
    public void marketIntent() {
        Intent intent = core.kakaoTalkMarketIntent(context);
        Uri uri = intent.getData();
        Assert.assertNotNull(uri);
        Assert.assertEquals("market", uri.getScheme());
        Assert.assertEquals("details", uri.getHost());
        Assert.assertEquals("com.kakao.talk", uri.getQueryParameter("id"));
        String referrer = uri.getQueryParameter("referrer");
        Assert.assertNotNull(referrer);
        try {
            JSONObject referrerJson = new JSONObject(referrer);
            Assert.assertEquals(configuration.getKAHeader(), referrerJson.get(CommonProtocol.KA_HEADER_KEY));
            Assert.assertEquals(configuration.getAppKey(), referrerJson.get(KakaoTalkLinkProtocol.APP_KEY));
            Assert.assertEquals(configuration.getAppVer(), referrerJson.get(KakaoTalkLinkProtocol.APP_VER));
            Assert.assertEquals(configuration.getPackageName(), referrerJson.get(KakaoTalkLinkProtocol.APP_PACKAGE));
        } catch (JSONException e) {
            Assert.fail("There was an exception parsing market referrer.");
        }
    }
}
